"""Copyright 2016-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""


from traceutils.trace_parser import TraceParser
from traceutils.oracle import Oracle
import argparse
import os.path
import sys


class TraceMatcher(argparse.ArgumentParser):
    """Traces matching component."""

    def main(self):
        """Matches traces that are read from command line arguments."""
        self.add_argument('file', nargs='*', help='path to trace file')
        args = self.parse_args()
        file_paths = args.file
        if not file_paths:
            self.error("List of input files is empty.")

        file_parsers = list()
        for file_path in file_paths:
            if os.path.isfile(file_path):
                file_parsers.append(TraceParser(file_path))
            else:
                self.error("Can't find file: %s\n" % file_path)

        oracle = Oracle()
        while self._has_record(file_parsers):
            for parser in file_parsers:
                if parser.has_next_record():
                    oracle.register_record(parser.next_record(), parser.get_trace())
                else:
                    file_parsers.remove(parser)

        oracle.check()

    def error(self, message):
        """Prints error message to stdout."""
        print >> sys.stderr, "Error: %s\n" % message
        self.print_help()
        sys.exit(2)

    @staticmethod
    def _has_record(parsers):
        """Checks whether at least one parser from the specified list can provide next record."""
        has_record = False

        for parser in parsers:
            if parser.has_next_record():
                has_record = True
                break

        return has_record

if __name__ == "__main__":
    sys.exit(TraceMatcher().main())
